package de.jottyfan.bico.modules.camp;

import static de.jottyfan.bico.db.camp.Tables.T_REGISTRATION;

import java.util.List;

import org.jooq.DSLContext;
import org.jooq.DeleteConditionStep;
import org.jooq.InsertValuesStep5;
import org.jooq.SelectConditionStep;
import org.jooq.UpdateConditionStep;
import org.jooq.impl.DSL;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import de.jottyfan.bico.Main;
import de.jottyfan.bico.db.camp.enums.EnumCamp;
import de.jottyfan.bico.db.camp.enums.EnumSex;
import de.jottyfan.bico.db.camp.tables.records.TRegistrationRecord;
import de.jottyfan.bico.modules.camp.model.RegistrationBean;
import jakarta.validation.Valid;

/**
 *
 * @author jotty
 *
 */
@Repository
public class RegistrationRepository {
	@Autowired
	private DSLContext jooq;

	/**
	 * save the validated bean
	 *
	 * @param bean        the bean
	 * @param registrator the principal name
	 */
	public void save(@Valid RegistrationBean bean, String registrator) {
		jooq.transaction(t -> {
			if (bean.getPkRegistration() == null) {
				InsertValuesStep5<TRegistrationRecord, EnumCamp, String, String, String, EnumSex> sql = DSL.using(t)
				// @formatter:off
					.insertInto(T_REGISTRATION,
							        T_REGISTRATION.CAMP,
							        T_REGISTRATION.REGISTRATOR,
							        T_REGISTRATION.FORENAME,
							        T_REGISTRATION.SURNAME,
							        T_REGISTRATION.SEX)
					.values(EnumCamp.Gemeindefreizeit_2025, registrator, bean.getForename(), bean.getSurname(), EnumSex.lookupLiteral(bean.getSex()));
				// @formatter:on
				Main.LOGGER.trace(sql);
				sql.execute();
			} else {
				UpdateConditionStep<TRegistrationRecord> sql = DSL.using(t)
				// @formatter:off
					.update(T_REGISTRATION)
					.set(T_REGISTRATION.FORENAME, bean.getForename())
					.set(T_REGISTRATION.SURNAME, bean.getSurname())
					.set(T_REGISTRATION.SEX, EnumSex.lookupLiteral(bean.getSex()))
					.set(T_REGISTRATION.REGISTRATOR, registrator)
					.where(T_REGISTRATION.PK_REGISTRATION.eq(bean.getPkRegistration()));
				// @formatter:on
				Main.LOGGER.trace(sql);
				sql.execute();
			}
		});
	}

	/**
	 * get all registrations of name
	 *
	 * @param name the name
	 * @return the registrations
	 */
	public List<RegistrationBean> getRegistrations(String name) {
		SelectConditionStep<TRegistrationRecord> sql = jooq
		// @formatter:off
			.selectFrom(T_REGISTRATION)
			.where(T_REGISTRATION.REGISTRATOR.eq(name));
		// @formatter:on
		Main.LOGGER.trace(sql);
		return sql.fetchInto(RegistrationBean.class);
	}

	/**
	 * get the registration of id if the creator is the name; null else
	 *
	 * @param id   the id
	 * @param name the name
	 * @return the bean or null
	 */
	public RegistrationBean getBean(Integer id, String name) {
		SelectConditionStep<TRegistrationRecord> sql = jooq
		// @formatter:off
			.selectFrom(T_REGISTRATION)
			.where(T_REGISTRATION.PK_REGISTRATION.eq(id))
			.and(T_REGISTRATION.REGISTRATOR.eq(name));
		// @formatter:on
		Main.LOGGER.trace(sql);
		return sql.fetchOneInto(RegistrationBean.class);
	}

	/**
	 * delete the registration if the name is the creator
	 *
	 * @param id   the id
	 * @param name the name
	 */
	public void delete(Integer id, String name) {
		DeleteConditionStep<TRegistrationRecord> sql = jooq
		// @formatter:off
			.deleteFrom(T_REGISTRATION)
			.where(T_REGISTRATION.PK_REGISTRATION.eq(id))
			.and(T_REGISTRATION.REGISTRATOR.eq(name));
		// @formatter:on
		Main.LOGGER.trace(sql);
		sql.execute();
	}
}
